#!/usr/bin/env bash
START_DIR=$(pwd)
APP_DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" >/dev/null 2>&1 && pwd )"

cleanup() {
    cd "${START_DIR}"
}
abort() {
    echo "ERROR"
    exit 1
}

trap 'cleanup' 0
trap 'abort' ERR

function check_binary_avail() {
    binary_name=$1
    software_name=$2
    if [ -z "$(which ${binary_name})" ]; then
        echo "ERROR: can't find ${software_name} installation (checked for ${binary_name} binary)"
        exit 1
    fi
}

Rbin="Rscript"
Ropts="--vanilla"

check_binary_avail ${Rbin} "R" # R
check_binary_avail "wish" "XQuartz or Tcl/Tk"

if [ ! -z "$1" ]; then
  CONFIG_PATH=${APP_DIR}/config.yml
  echo "default:" > ${CONFIG_PATH}
  echo "  lang: $1" >> ${CONFIG_PATH}
fi

# paths relative to ${APP_DIR}
RScriptFile="utils/package_manager.R"
sLogPath="log"
oConfigFile="utils/config.cfg"

# echo "${APP_DIR}"
cd "${APP_DIR}"
sLogFile=$(grep "error_log" ${oConfigFile} | cut -f 2 -d ":" | tr -d " " | tr -d \")
# echo ${sLogFile}
if [ -z "${sLogFile}" ]; then
    sLogFile="error.log" # relative to ${sLogPath}
fi
if [ ! -d "${sLogPath}" ]; then
    mkdir -p ${sLogPath}
fi
Outfile="${sLogPath}/${sLogFile}";
if [ ! -e "${Outfile}" ]; then
    touch "${Outfile}"
fi
# echo ${Outfile}

strCommand="${Rbin} ${Ropts} ${RScriptFile} >${Outfile} 2>&1"
# echo "${strCommand}"
eval ${strCommand}
