# ui.R

library(shinyjs)
#library(shinysky)

shinyUI(
  fluidPage(theme = "ecoval.css",

    useShinyjs(),  # Set up shinyjs
    navbarPage(uiText$navbar.title,



################################
## Tab: Home
      tabPanel(uiText$navbar.home,
        titlePanel(uiText$home.title),
        tags$p(tags$strong(uiText$home.version.info)),

        tags$h3(paste0(uiText$purpose,":")),
        tags$p(tags$strong(uiText$purpose.text)),
        tags$p(uiText$current.modules.lakes),
        tags$ul(
          tags$li(uiText$module.lakeshoremorph)
        ),
        tags$p(uiText$current.modules.rivers),
        tags$ul(
          tags$li(uiText$module.macrophytes)
        ),

        tags$h3(paste0(uiText$references,":")),
        tags$ul(
          tags$li(tags$a("http://www.modul-stufen-konzept.ch"))
        ),

        tags$h3(paste0(uiText$contact.person,":")),
        tags$ol(
          tags$li("Peter Reichert <peter.reichert@eawag.ch>"),
          tags$li("Nele Schuwirth <nele.schuwirth@eawag.ch>")
        ),

        tags$div(id="footer", tags$p(
          "Copyright (C) 2017-2020",
          tags$a(href = "https://sis.id.ethz.ch", "SIS ID ETH Zurich"),
          ", ",
          tags$a(href = "https://www.eawag.ch/en/department/siam/", "SIAM Eawag")))
      ),



################################
## Tab: Lakes
      tabPanel(uiText$lake.shore,
        titlePanel(uiText$lakes.title.panel),
          fluidRow(
          column(4,
            wellPanel(

              tags$h3(uiText$definitions),
              selectInput("lakesLanguage",
                paste0(uiText$language.encoding,":"),
                lakesAPI$dictionaryLanguages$choices,
                selected=lakesAPI$dictionaryLanguages$default),

              tags$h3(uiText$input.data),
              directoryInput('lakesInputDir',
                btn.label=uiText$dir.input),
              div(class="narrow",
                # this input file separator is only for info - it isn't even
                # passed to the lakesAPI$readInputFiles fun call
                disabled(textInput("lakesInputSeparator",
                  label = paste0(uiText$separator,":"),
                  value = ";"))
              ),
              tags$h4(uiText$column.names),
              textInput("lakesColnameStartPoint",
                label = paste0(uiText$csv.col.start,"*:"),
                value = uiText$csv.col.start.value),
              textInput("lakesColnameEndPoint",
                label = paste0(uiText$csv.col.end,"*:"),
                value = uiText$csv.col.end.value),
              textInput("lakesColnameAttributeLevel",
                label = paste0(uiText$csv.col.level,"*:"),
                value = uiText$csv.col.level.value),
              textInput("lakesColnameMeasureId",
                label = paste0(uiText$csv.col.measureId,":"),
                value = uiText$csv.col.measureId.value),
              textInput("lakesColnameComments",
                label = paste0(uiText$csv.col.comments,":"),
                value = uiText$csv.col.comments.value),


              tags$h3(uiText$output.data),
              directoryInput('lakesOutputDir',
                btn.label=uiText$dir.output),
                div(class="narrow",
                  textInput("lakesOutputSeparator",
                  label = paste0(uiText$separator,":"),
                  value = ";")
                ),

              # TODO: fix input names - english and prefixed w/ "lakes"
              textInput("attributeFileName",
                label = paste0(uiText$attributes," [",lakesAPI$defaultFileExtensions$output.file.attributes,"]:"),
                value = uiText$attributes),
              textInput("bewertungenFileName",
                label = paste0(uiText$assesments," [",lakesAPI$defaultFileExtensions$output.file.assesments,"]:"),
                value = uiText$assesments),
              textInput("hierarchienFileName",
                label = paste0(uiText$evaluation.hierarchies," [",lakesAPI$defaultFileExtensions$output.file.evaluation.hierarchies,"]:"),
                value = uiText$target.hierarchies),
              fluidRow(
                column(3, offset = 2,
                      textInput("hierarchienBreite",
                      label = paste0(uiText$w.idth,":"), #uiText$width,
                      value = "20")
                ),
                column(3, offset = 2,
                      textInput("hierarchienHohe",
                      label = paste0(uiText$h.eight,":"), #uiText$height,
                      value = "12")
                )
              ),
              # fluidRow(
              #   column(10, offset = 2,
              #     sliderInput("hierarchienSlider",
              #       label = uiText$range,
              #       min = 0, max = 0,
              #       value = c(0, 0))
              #   )
              # ),
              HTML("&nbsp;"),
              textInput("bewertungsLinienFileName",
                label = paste0(uiText$evaluation.lines," [",lakesAPI$defaultFileExtensions$output.file.evaluation.lines,"]:"),
                value = uiText$evaluation.lines),
              fluidRow(
                column(3, offset = 2,
                      textInput("bewertungsLinienBreite",
                      label = paste0(uiText$w.idth,":"), #uiText$width,
                      value = "20")
                ),
                column(3, offset = 2,
                      textInput("bewertungsLinienHohe",
                      label = paste0(uiText$h.eight,":"), #uiText$height,
                      value = "12")
                )
              )#,
              # fluidRow(
              #   column(10, offset = 2,
              #     sliderInput("bewertungsLinienSlider",
              #     label = uiText$range,
              #     min = 0, max = 0,
              #     value = c(0, 0))
              #   )
              # )
            ) # wellPanel
          ), # column


          column(8,

            tags$h3(uiText$evaluation),
            fluidRow(
              column(6, offset=3,
                shiny::actionButton("lakesReadFilesAndEvaluateAction",
                  label = paste(uiText$read.files, uiText$and, uiText$evaluate)),
                hidden(tags$div(id="lakesReadFilesAndEvaluateRunning",
                    class="alert alert-info processing",
                    tags$p(paste0(uiText$analysis.running,"...")))),
                htmlOutput("lakesReadFilesAndEvaluateOutput")
              )
            ),

            tags$h3(uiText$show.results),
            fluidRow(
              column(3,
                htmlOutput("lakesAttributeOutput")
              ),
              column(3,
                htmlOutput("lakesAssesmentsOutput")
              ),
              column(3,
                htmlOutput("lakesHierarchiesOutput")
              ),
              column(3,
                htmlOutput("lakesEvaluationLinesOutput")
              )
            ),


            tags$div( # <div> needed to limit overlay
              tags$h3(uiText$interactive.plots),
              tags$div(class="overlayRight",
                  selectizeInput(inputId = "lakesSegmentName",
                    label = paste0(uiText$segment,":"),
                    choices = NULL)
              )
            ),
            tabsetPanel(
              tabPanel(uiText$single.hierarchy,

                fluidRow(
                  column(1, offset = 0,
                    shiny::actionButton("lakesClearModifications",
                      label = "", icon = icon("undo"),
                      style='max-width: 30px; min-width: 30px;')
                  ),
                  column(3, offset = 0,
                    selectizeInput(inputId = "lakesSingleHierarchyAttribute",
                      label = paste0(uiText$attribute,":"),
                      choices = NULL)
                  ),
                  column(2, offset = 1,
                    selectizeInput("lakesSingleHierarchyAttributeValue",
                      label = "= ", #paste0(uiText$level,":"),
                      choices= NULL),
                    # report here issues w/ attrib. or value
                    htmlOutput("lakesSingleHierarchyAttributeValueOutput")
                  ),
                  column(3, offset = 2,
                    checkboxInput("lakesIfSingleHierarchyPlotWithAttrib",
                      label = uiText$show.attributes,
                      value = FALSE)
                  )
                ),

                htmlOutput("lakesSingleHierarchyOutput"),
                # NOTE: wrapper div height calculated in ecoval.css as 100vh minus
                #       absolute height of content above plot in px;
                #       on Win10 might not work in R Studio Browser, and in IE11
                #       works with "id" atrribute, but not with "class"
                tags$div(id="interactive-plot-lakes",
                  plotOutput("lakesSingleHierarchyPlot", height="100%")
                )
              )
            )
          ) # column
        ) # fluidRow
      ), # tabPanel



################################
## Tab: Macrophytes
      tabPanel(uiText$macrophytes,
        titlePanel(uiText$macrophytes.title),
        fluidRow(
          column(4,
            wellPanel(

              tags$h3(uiText$definitions),
              selectInput("macrophytesLanguage",
                paste0(uiText$language.encoding,":"),
                macrophytesAPI$dictionaryLanguages$choices,
                selected=macrophytesAPI$dictionaryLanguages$default),
              selectInput("macrophytesSamplingProtocol",
                paste0(uiText$sampling.protocol,":"),
                macrophytesAPI$samplingProtocols$choices,
                selected=macrophytesAPI$samplingProtocols$default),
              numericInput("macrophytesNSamples",
                label = paste0(uiText$samples.for.uncertainty,":"),
                value = macrophytesAPI$defaultSampleSize, min=0, step=100),


              tags$h3(uiText$input.data),
              directoryInput('macrophytesInputDataDir',
                btn.label = uiText$dir.data),
              directoryInput('macrophytesInputImgDir',
                btn.label = uiText$dir.images),
              div(class="narrow",
                textInput("macrophytesInputSeparator",
                  label = paste0(uiText$separator," [",uiText$tab,"]:"),
                  value = "")
              ),

              tags$h4(uiText$filenames),
              selectizeInput(inputId = "macrophytesSitesFileName",
                  label = paste0(uiText$sites.data,"*:"),
                  choices = NULL),
              selectizeInput(inputId = "macrophytesSpeciesFileName",
                  label = paste0(uiText$species.data,":"),
                  choices = NULL),
              selectizeInput(inputId = "macrophytesPlausibilityFileName",
                  label = paste0(uiText$plausibilised.river.types,":"),
                  choices = NULL),


              tags$h3(uiText$output.data),
              directoryInput('macrophytesOutputDir',
                btn.label = uiText$dir.output),
              div(class="narrow",
                textInput("macrophytesOutputSeparator",
                  label = paste0(uiText$separator," [",uiText$tab,"]:"),
                  value = "")
              ),

              tags$h4(uiText$filenames),
              textInput("macrophytesNumResFileName",
                label = paste0(uiText$numeric.results," [",macrophytesAPI$defaultFileExtensions$output.file.numeric,"]:"),
                value = uiText$filename.res.num),
              textInput("macrophytesDocFileName",
                label = paste0(uiText$documentation, " [",macrophytesAPI$defaultFileExtensions$output.file.doc,"]:"),
                value = uiText$filename.res.doc),
              textInput("macrophytesUsedTaxaFileName",
                label = paste0(uiText$used.taxa, " [",macrophytesAPI$defaultFileExtensions$output.file.taxa.used,"]:"),
                value = uiText$filename.used.taxa),
              textInput("macrophytesDiscardedTaxaFileName",
                label = paste0(uiText$discarded.taxa, " [",macrophytesAPI$defaultFileExtensions$output.file.taxa.removed,"]:"),
                value = uiText$filename.discarded.taxa),
              textInput("macrophytesQualityChecksFileName",
                label = paste0(uiText$quality.checks, " [",macrophytesAPI$defaultFileExtensions$output.file.quality.checks,"]:"),
                value = uiText$filename.quality.checks)

            ) # wellPanel
          ), # column



          column(8,
            #shinysky::busyIndicator(text=uiText$please.wait, wait=2000),
            tags$h3(uiText$evaluation_s_),
            checkboxInput("macrophytesIfWriteDoc",
              label = paste(uiText$incl, uiText$sites.documentation,
                  paste0("[",macrophytesAPI$defaultFileExtensions$output.file.doc,"]")
                ),
              value = FALSE),
            fluidRow(
              column(3,
                shiny::actionButton("macrophytesCheckFilesAction",
                  label = uiText$check.files),
                hidden(tags$div(id="macrophytesCheckFilesRunning",
                    class="alert alert-info processing",
                    tags$p(paste0(uiText$analysis.running,"...")))),
                htmlOutput("macrophytesCheckFilesOutput")
              ),
              column(3,
                shiny::actionButton("macrophytesComputeTypificationAction",
                  label = uiText$compute.typification),
                hidden(tags$div(id="macrophytesComputeTypificationRunning",
                    class="alert alert-info processing",
                    tags$p(paste0(uiText$analysis.running,"...")))),
                htmlOutput("macrophytesComputeTypificationOutput")
              ),
              column(3,
                shiny::actionButton("macrophytesEvaluatePrelimAction",
                  label = paste(uiText$assesment,uiText$prelim)),
                hidden(tags$div(id="macrophytesEvaluatePrelimRunning",
                    class="alert alert-info processing",
                    tags$p(paste0(uiText$analysis.running,"...")))),
                htmlOutput("macrophytesEvaluatePrelimOutput")
              ),
              column(3,
                shiny::actionButton("macrophytesEvaluatePlausAction",
                  label = paste(uiText$assesment,uiText$plaus)),
                hidden(tags$div(id="macrophytesEvaluatePlausRunning",
                    class="alert alert-info processing",
                    tags$p(paste0(uiText$analysis.running,"...")))),
                htmlOutput("macrophytesEvaluatePlausOutput")
              )
            ),

            tags$h3(uiText$show.results),
            fluidRow(
              column(2,
                htmlOutput("macrophytesNumResOutput")
              ),
              column(3,
                htmlOutput("macrophytesUsedTaxaOutput")
              ),
              column(3,
                htmlOutput("macrophytesDiscardedTaxaOutput")
              ),
              column(2,
                htmlOutput("macrophytesQualityChecksOutput")
              ),
              column(2,
                htmlOutput("macrophytesDocumentationOutput")
              )
            ),


            # Note: this looks confusing
            # fluidRow(
            #   column(9,
            #     tags$h3(uiText$interactive.plots)
            #   ),
            #   column(3,
            #     selectizeInput(inputId = "macrophytesObservationCode",
            #       label = paste0(uiText$observation.code,":"),
            #       choices = NULL)
            #   )
            # ),
            tags$div( # <div> needed to limit overlay
              tags$h3(uiText$interactive.plots),
              tags$div(class="overlayRight",
                selectizeInput(inputId = "macrophytesObservationCode",
                  label = paste0(uiText$observation.code,":"),
                  choices = NULL)
              )
            ),

            tabsetPanel(
              tabPanel(uiText$typification,

                fluidRow(
                  column(1,
                    shiny::actionButton("macrophytesTypificationClearModifications",
                      label = "", icon = icon("undo"),
                      style='max-width: 30px; min-width: 30px;')
                  ),
                  column(3, offset = 0,
                    selectizeInput(inputId = "macrophytesTypificationAttribute",
                      label = paste0(uiText$attribute,":"),
                      choices = NULL)
                  ),
                  column(2, offset = 1,
                    numericInput("macrophytesTypificationAttributeValue",
                      label = "= ", #paste0(uiText$level,":"),
                      0, min=0, step=0.01),
                    # report here issues w/ attrib. or value
                    htmlOutput("macrophytesTypificationAttributeValueOutput")
                  )
                ),

                # plot type
                radioButtons("macrophytesTypificationPlotChoice", "",
                  choiceNames = list(uiText$types, uiText$growth.habit.group),
                  choiceValues = list(uiText$types, uiText$growth.habit.group),
                  inline = TRUE),

                htmlOutput("macrophytesTypificationMessageOutput"),
                # NOTE: wrapper div height calculated in ecoval.css as 100vh minus
                #       absolute height of content above plot in px
                #       on Win10 might not work in R Studio Browser, and in IE11
                #       works with "id" atrribute, but not with "class"
                tags$div(id="interactive-plot-macrophytes-1",
                  plotOutput("macrophytesTypificationPlot", height="100%")
                )
              ),

              tabPanel(uiText$assesment,

                fluidRow(
                  column(1,
                    shiny::actionButton("macrophytesAssesmentClearModifications",
                      label = "", icon = icon("undo"),
                      style='max-width: 30px; min-width: 30px;')
                  ),
                  column(3, offset=0,
                    selectizeInput(inputId = "macrophytesAssesmentAttribute",
                      label = paste0(uiText$attribute,":"),
                      choices = NULL)
                  ),
                  column(2, offset=1,
                    numericInput("macrophytesAssesmentAttributeValue",
                      label = "= ", #paste0(uiText$level,":"),
                      0, min=0, step=0.01),
                    # report here issues w/ attrib. or value
                    htmlOutput("macrophytesAssesmentAttributeValueOutput")
                  ),
                  column(3, offset=1,
                    selectizeInput(inputId = "macrophytesAssesmentRiverType",
                      label = paste0(uiText$river.type,":"),
                      choices = NULL)
                  )
                ),

                # plot type
                fluidRow(
                  column(9,
                    radioButtons("macrophytesAssesmentPlotChoice", "",
                      choiceNames = list(uiText$orig, uiText$plaus),
                      choiceValues = list(uiText$orig, uiText$plaus),
                      inline = TRUE)
                  ),
                  column(3,
                    checkboxInput("macrophytesIfAssesmentPlotWithAttrib",
                      label = uiText$show.attributes,
                      value = FALSE)
                  )
                ),
                htmlOutput("macrophytesAssesmentMessageOutput"),
                # NOTE: wrapper div height calculated in ecoval.css as 100vh minus
                #       absolute height of content above plot in px
                #       on Win10 might not work in R Studio Browser, and in IE11
                #       works with "id" atrribute, but not with "class"
                tags$div(id="interactive-plot-macrophytes-2",
                  plotOutput("macrophytesAssesmentPlot", height="100%")
                )
              )
            )
          ) # column
        ) # fluidRow
      ) # tabPanel

    ) # navbarPage
  ) # fluidPage
) # shinyUI
