pb <- NULL
if (exists("winProgressBar")) {
  initProgressBar <- winProgressBar
  setProgressBar <- setWinProgressBar
} else {
  initProgressBar <- tcltk::tkProgressBar
  setProgressBar <- tcltk::setTkProgressBar
}
# Use tryCatch to display error messages in config$logging$filename
appexit_msg <- tryCatch({
  pb <- initProgressBar(
    title = "Initializing ...",
    label = "Internet connection might be required ...")
  Sys.sleep(1)

  # capture the current working directory
  # set the package search path to the app specific library
  appwd <- getwd()
  applibpath <- file.path(appwd, "library")

  # Create app/library if it doesn't exist (e.g. first run)
  if (!dir.exists(applibpath)) dir.create(applibpath)

  .libPaths(c(applibpath, .libPaths()))

  # Load functions to ensure software dependencies and check the internet
  # Attention: do it only after .libPaths() have been set
  source("utils/ensure.R")

  message("library paths:\n", paste0("... ", .libPaths(), collapse = "\n"))
  message("working path:\n", paste("...", appwd))
  message("interactive R session:\n", paste("...", interactive()))

  # Initialize RInno
  # Note: keep packages R version dependency in sync with min. R version
  #       required by the  function
  init_pkgs <- c(devtools = '>=1.13.6', jsonlite = '>=1.5', httr = '>=1.3.1')
  setProgressBar(pb, 0, label = "Ensuring RInno dependencies ...")
  repo <- "http://stat.ethz.ch/CRAN/" #getCRANmirrors(local.only = TRUE)[118,]$URL
  ._ <- mapply(ensure, init_pkgs, names(init_pkgs), MoreArgs = list(pkgs = init_pkgs, check_connection = FALSE, repo = repo))

  # Read the application config
  library("jsonlite", character.only = TRUE)
  library("devtools", character.only = TRUE)
  library("httr", character.only = TRUE)
  config_path <- file.path(appwd, "utils/config.cfg")
  if (!file.exists(config_path))
    stop("config file not found at ", config_path)
  config <- jsonlite::fromJSON(config_path)
  message("Read config:\n", jsonlite::toJSON(config, pretty = TRUE))

  # Package dependency list
  pkgs <- config$pkgs$pkgs; remotes <- config$remotes; locals <- config$locals$pkgs

  close(pb)
  # Provide some initialization status updates
  pb <- initProgressBar(
    title = sprintf("Starting %s ...", config$appname),
    label = "Internet connection might be required ...")
  Sys.sleep(1)

  # If an app repository has been provided, install the app from there
  if (config$app_repo[[1]] != "none") {
    source("utils/get_app_from_app_url.R")
  }

  # ensure all package dependencies are installed
  message("ensuring packages: ", paste(names(pkgs), collapse = ", "))
  setProgressBar(pb, 0, label = "Ensuring package dependencies ...")
  ._ <- mapply(ensure, pkgs, names(pkgs), MoreArgs = list(pkgs = pkgs))
  if (remotes[1] != "none") {
    message("ensuring remotes: ", paste(remotes, collapse = ", "))
    setProgressBar(pb, 0, label = "Ensuring remote package dependencies ...")
    ._ <- lapply(remotes, ensure_remotes)
  }
  if (locals[1] != "none") {
    message("ensuring locals: ", paste(names(locals), collapse = ", "))
    setProgressBar(pb, 0, label = "Ensuring local package dependencies ...")
    ._ <- mapply(ensure_local, locals, names(locals))
  }

  for (i in seq_along(pkgs)) {
    setProgressBar(pb,
                      value = i / (length(pkgs) + 1),
                      label = sprintf("Loading package - %s", names(pkgs)[i]))

    library(names(pkgs)[i], character.only = TRUE)
  }

  setProgressBar(pb, 1.00, label = "Starting application")
  close(pb)

  source(file.path(appwd, "utils/app.R"))

  "application terminated normally"
},
error = function(e) {
  msg <- sprintf("Startup failed with error(s):\n\n%s", e$message)
  tcltk::tk_messageBox(
    type = "ok",
    message = msg,
    icon = "error")
  msg
},
finally = {
  if (!is.null(pb)) close(pb)
})

message(appexit_msg)
